@@ -1,20 +1,18 @@
 /* word_io.c -- word oriented I/O
    Copyright (C) 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -24,6 +22,7 @@
 #include <errno.h>
 #include <stdbool.h>		/* for bool */
 #include <assert.h>
+#include <stdlib.h>
 
 #include "error.h"
 #include "quote.h"
@@ -142,9 +141,10 @@ getword (FILE *fp,
        * Either condition is fatal.
        */
       if (feof(fp))
-	error(1, 0, _("Unexpected EOF in %s"), quoted_name);
-      else 
-	error(1, errno, "error reading a word from %s", quoted_name);
+	error(1, 0, _("unexpected EOF in %s"), quoted_name);
+      else
+	error(1, errno, _("error reading a word from %s"), quoted_name);
+      abort ();
     }
   else
     {
@@ -162,7 +162,7 @@ putword (FILE *fp, int word,
   /* You must decide before calling this function which 
    * endianness you want to use. 
    */
-  assert(endian_state_flag != GetwordEndianStateInitial);
+  assert (endian_state_flag != GetwordEndianStateInitial);
   if (GetwordEndianStateSwab == endian_state_flag)
     {
       word = bswap_32(word);
@@ -174,4 +174,3 @@ putword (FILE *fp, int word,
   else
     return false;
 }
-
